#!@PERL@

# --8<--8<--8<--8<--
#
# Copyright (C) 2010 Smithsonian Astrophysical Observatory
#
# This file is part of make-world
#
# make-world is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# make-world is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
#       Free Software Foundation, Inc.
#       51 Franklin Street, Fifth Floor
#       Boston, MA  02110-1301, USA
#
# -->8-->8-->8-->8--


use strict;
use warnings;

package MakeWorld::Log;

use MakeWorld::IORedir;

use base qw( Exporter );

use constant Notice => 1;
use constant Info   => 2;
use constant Debug  => 3;

our @EXPORT = qw( msg msgp );

my $log_level = 1;

my @prefix;
my $prefix = '';

sub new {

    my $class = shift;
    &push(@prefix, shift);

    return bless {}, $class;
}

sub DESTROY {
    &pop;
}

sub mkpfx
{
    $prefix = join( '', map { "$_: " } @prefix );
}

sub push
{
    push @prefix, shift;
    mkpfx();
}

sub pop
{
    pop @prefix;
    mkpfx();
}

sub level
{
    my $old = $log_level;
    $log_level = $_[0] if @_;
    return $old;
}

sub msgp {

    my $pfx = shift;
    &push( $pfx );
    eval { msg(@_) };
    &pop;
}

sub msg {
    my $msg_level = shift;

    if ( $msg_level <= $log_level )
    {
	my $rd = MakeWorld::IORedir->new( reset => 1 );
	for my $msg ( @_ )
	{
	    if ( ! ref $msg )
	    {
		print "$prefix$_\n" foreach split /\n/, $msg;
	    }
	    elsif ( 'ARRAY' eq ref $msg )
	    {
		print "$prefix\[$_]\n" foreach @$msg;
	    }
	    elsif ( 'HASH' eq ref $msg )
	    {
		print qq[$prefix$_ = "$msg->{$_}"\n] foreach sort keys %$msg;
	    }

	}

    }
}


1;
